@@ -19,25 +19,9 @@
 
 #include <vector>
 
-#if defined(__MINGW32__)
-#undef Realloc
-#undef Free
-#endif
+#include "RcppAnnoy.h"
 
-#define __ERROR_PRINTER_OVERRIDE__ REprintf
-
-#include <annoylib.h>
-#include <kissrandom.h>
-
-#include "uwot/matrix.h"
-
-#if defined __RcppAnnoy_0_16_2__
-#ifdef ANNOYLIB_MULTITHREADED_BUILD
-  typedef AnnoyIndexMultiThreadedBuildPolicy AnnoyIndexThreadedBuildPolicy;
-#else
-  typedef AnnoyIndexSingleThreadedBuildPolicy AnnoyIndexThreadedBuildPolicy;
-#endif
-#endif
+typedef AnnoyIndexSingleThreadedBuildPolicy AnnoyIndexThreadedBuildPolicy;
 
 struct UwotAnnoyEuclidean {
   using Distance = Euclidean;
@@ -74,11 +58,8 @@ template <typename UwotAnnoyDistance> struct NNWorker {
   std::vector<typename UwotAnnoyDistance::T> dists;
 
   AnnoyIndex<typename UwotAnnoyDistance::S, typename UwotAnnoyDistance::T,
-#if defined __RcppAnnoy_0_16_2__
-             typename UwotAnnoyDistance::Distance, Kiss64Random, AnnoyIndexThreadedBuildPolicy>
-#else
-             typename UwotAnnoyDistance::Distance, Kiss64Random>
-#endif
+             typename UwotAnnoyDistance::Distance, Kiss64Random,
+             AnnoyIndexThreadedBuildPolicy>
       index;
 
   NNWorker(const std::string &index_name, const std::vector<double> &mat,
@@ -88,15 +69,15 @@ template <typename UwotAnnoyDistance> struct NNWorker {
         idx(nrow * n_neighbors, -1), dists(nrow * n_neighbors), index(ncol) {
     index.load(index_name.c_str());
   }
-  
-  ~NNWorker() { 
-    index.unload();
-  }
+
+  ~NNWorker() { index.unload(); }
 
   void operator()(std::size_t begin, std::size_t end) {
     for (auto i = begin; i < end; i++) {
       std::vector<typename UwotAnnoyDistance::T> fv(ncol);
-      uwot::get_row(mat, nrow, ncol, i, fv);
+      for (std::size_t j = 0; j < ncol; j++) {
+        fv[j] = mat[i + j * nrow];
+      }
 
       std::vector<typename UwotAnnoyDistance::S> result;
       std::vector<typename UwotAnnoyDistance::T> distances;
@@ -108,8 +89,8 @@ template <typename UwotAnnoyDistance> struct NNWorker {
       }
 
       for (std::size_t j = 0; j < n_neighbors; j++) {
-        uwot::set_row(dists, nrow, n_neighbors, i, distances);
-        uwot::set_row(idx, nrow, n_neighbors, i, result);
+        dists[i + j * nrow] = distances[j];
+        idx[i + j * nrow] = result[j];
       }
     }
   }
